# Zope
from OFS import SimpleItem
from AccessControl import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
import Globals
from Persistence import Persistent
from DateTime import DateTime

# Silva
from Products.Silva.interfaces import IMember, IMemberService
from Products.Silva.Membership import Member, CachedMember, NoneMember, cloneMember
from Products.Silva import SilvaPermissions
from Products.Silva.helpers import add_and_edit
from Products.Silva.Security import Security

class CachedLDAPMember(CachedMember):
    """Cached LDAP member
    
        wrapper around CachedMember with some ETH specific stuff
    """

    def extra(self, field):
        if field == 'userid':
            return self.userid()

class LDAPMember(Member, Security, SimpleItem.SimpleItem):
    """LDAP member object, specific to the ETH setup"""
    __implements__ = IMember

    security = ClassSecurityInfo()

    meta_type = 'Silva ETH LDAP Member'

    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                              'security_trigger')
    def security_trigger(self):
        """Triggers the security machinery"""
        pass
    
    def __init__(self, id):
        self.id = id
        self._fullname = None
        self._email = None
        self._email_from_LDAP = None
        self._departments = None
        self._editor = 'field_editor'
        
    def manage_afterAdd(self, *args, **kwargs):
        LDAPMember.inheritedAttribute('manage_afterAdd')(self, *args, **kwargs)
        self._sync()
        
    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                              'set_fullname')
    def set_fullname(self, fullname):
        self._fullname = fullname

    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                              'set_email')
    def set_email(self, email):
        self._email = email

    security.declareProtected(SilvaPermissions.ChangePermissions,
                              'set_departments')
    def set_departments(self, departments):
        self._departments = departments

    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                              'set_editor')
    def set_editor(self, editor):
        self._editor = editor
        
    # ACCESSORS

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'userid')
    def userid(self):
        """userid
        """
        return self.id
    
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'fullname')
    def fullname(self):
        """fullname
        """
        return self._fullname

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'email')
    def email(self):
        """email

            if there's an _email field on self, consider it overwritten
            from LDAP, if not return the LDAP content
        """
        return self._email or self._email_from_LDAP
        
    def overridden_email(self):
        """email as stored on this object (doesn't do LDAP search)"""
        return self._email

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'departments')
    def departments(self):
        """departments
        """
        return self._departments

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'is_approved')
    def is_approved(self):
        """is_approved
        """
        return 1

    def extra(self, field):
        if field == 'userid':
            return self.userid()

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'editor')
    def editor(self):
        return self._editor

    def _sync(self):
        info = self.service_members._get_ldap_info(self.id)
        if info is None:
            return
        self._fullname = info.getProperty('gecos')
        self._email_from_LDAP = info.getProperty('email')

        return info

    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                              'manage_sync')
    def manage_sync(self, REQUEST=None):
        """sync the object with LDAP"""
        return self._sync()
            
Globals.InitializeClass(LDAPMember)

manage_addLDAPMemberForm = PageTemplateFile(
    "www/ldapMemberAdd", globals(),
    __name__='manage_addLDAPMemberForm')

def manage_addLDAPMember(self, id, REQUEST=None):
    """Add a LDAP Member."""
    object = LDAPMember(id)
    self._setObject(id, object)
    object.sec_assign(id, 'ChiefEditor')
    add_and_edit(self, id, REQUEST)
    return ''

class LDAPMemberService(SimpleItem.SimpleItem):
    __implements__ = IMemberService

    security = ClassSecurityInfo()

    meta_type = 'Silva ETH LDAP Member Service'
    
    def __init__(self, id):
        self.id = id

    def _get_ldap_info(self, userid):
        user_folder = self.aq_inner.acl_users
        # this uses the cache in LDAPUserFolder, returns None if user
        # unknown (but does give an ldap hit in that case..)
        return user_folder.getUser(userid)
    
    security.declareProtected(SilvaPermissions.ApproveSilvaContent,
                              'find_members')
    def find_members(self, search_string):
        try:
            search_string = unicode(search_string, 'cp1252').encode('ascii')
        except UnicodeError:
            return []
        user_folder = self.aq_inner.acl_users
        res = user_folder.findUser('gecos', search_string)
        res.sort(self._sort_results)
        ret = []
        for dict in res:
            # try to get the info from a memberobject if that exists, since
            # it may contain overridden values
            # XXX This currently doesn't use API functionality to do this, 
            # perhaps an 'is_member' method would be in place?
            members = self.Members.aq_inner
            member = getattr(members, dict['uid'], None)
            if member is not None:
                ret.append(CachedLDAPMember(
                    userid=dict['uid'],
                    fullname=dict['gecos'],
                    email=member.overridden_email() or dict.get('mail'),
                    is_approved=member.is_approved(),
                    editor=member.editor()
                    ).__of__(self))
            else:
                ret.append(CachedLDAPMember(
                    userid=dict['uid'],
                    fullname=dict['gecos'],
                    email=dict.get('mail'),
                    is_approved=0,
                    editor='field_editor'
                    ).__of__(self))
        return ret

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'is_user')
    def is_user(self, userid):
        # check whether user is in Members folder
        # XXX this means users will be reported as existing even if they
        # have been removed from LDAP..it does gain important
        # performance after startup, though..
        if hasattr(self.Members.aq_base, userid):
            return 1
        # check whether user is in ldap
        if self._get_ldap_info(userid):
            return 1
        # try to fall back on non-ldap
        above_root = self.get_root().aq_parent
        return userid in above_root.get_valid_userids()
    
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'get_member')
    def get_member(self, userid):
        if not self.is_user(userid):
            return None
        # get member, add it if it doesn't exist yet
        members = self.Members.aq_inner
        member = getattr(members, userid, None)
        if member is None:
            members.manage_addProduct['SilvaExtETHLDAP'].manage_addLDAPMember(userid)
            member = getattr(members, userid)
        return member

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'get_cached_member')
    def get_cached_member(self, userid):
        return cloneMember(self.get_member(userid)).__of__(self)

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'allow_authentication_requests')
    def allow_authentication_requests(self):
        return 0

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'allow_subscription')
    def allow_subscription(self):
        """Available for backward compatibility
        
        Is replaced by 'allow_authentication_requests'
        """
        return 0

    def _sort_results(self, a, b):
        ret = 0
        if a.has_key('sn') and b.has_key('sn'):
            ret = cmp(a['sn'], b['sn'])
        if ret == 0:
            ret = cmp(a['gecos'], b['gecos'])
        if ret == 0:
            ret = cmp(a['uid'], b['uid'])
        return ret


    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'get_extra_names')
    def get_extra_names(self):
        return ['userid']
    
Globals.InitializeClass(LDAPMemberService)

manage_addLDAPMemberServiceForm = PageTemplateFile(
    "www/ldapMemberServiceAdd", globals(),
    __name__='manage_addLDAPMemberServiceForm')

def manage_addLDAPMemberService(self, id, REQUEST=None):
    """Add a LDAP Member Service."""
    object = LDAPMemberService(id)
    self._setObject(id, object)
    add_and_edit(self, id, REQUEST)
    return ''
